@@ -17,24 +17,19 @@
 """
 First run wizard
 """
-import os
 import logging
-import json
 import random
 
 from functools import partial
 
 from PySide import QtCore, QtGui
-from twisted.internet import threads
 
+from leap.bitmask.config import flags
 from leap.bitmask.config.leapsettings import LeapSettings
-from leap.bitmask.config.providerconfig import ProviderConfig
-from leap.bitmask.crypto.srpregister import SRPRegister
-from leap.bitmask.provider.providerbootstrapper import ProviderBootstrapper
 from leap.bitmask.services import get_service_display_name, get_supported
-from leap.bitmask.util.request_helpers import get_content
+from leap.bitmask.util.credentials import password_checks, username_checks
+from leap.bitmask.util.credentials import USERNAME_REGEX
 from leap.bitmask.util.keyring_helpers import has_keyring
-from leap.bitmask.util.password import basic_password_checks
 
 from ui_wizard import Ui_Wizard
 
@@ -53,14 +48,15 @@ class Wizard(QtGui.QWizard):
     REGISTER_USER_PAGE = 4
     SERVICES_PAGE = 5
 
-    BARE_USERNAME_REGEX = r"^[A-Za-z\d_]+$"
-
-    def __init__(self, bypass_checks=False):
+    def __init__(self, backend, bypass_checks=False):
         """
         Constructor for the main Wizard.
 
+        :param backend: Backend being used
+        :type backend: Backend
         :param bypass_checks: Set to true if the app should bypass
-        first round of checks for CA certificates at bootstrap
+                              first round of checks for CA
+                              certificates at bootstrap
         :type bypass_checks: bool
         """
         QtGui.QWizard.__init__(self)
@@ -86,23 +82,13 @@ class Wizard(QtGui.QWizard):
         self.ui.btnCheck.clicked.connect(self._check_provider)
         self.ui.lnProvider.returnPressed.connect(self._check_provider)
 
-        self._provider_bootstrapper = ProviderBootstrapper(bypass_checks)
-        self._provider_bootstrapper.name_resolution.connect(
-            self._name_resolution)
-        self._provider_bootstrapper.https_connection.connect(
-            self._https_connection)
-        self._provider_bootstrapper.download_provider_info.connect(
-            self._download_provider_info)
-
-        self._provider_bootstrapper.download_ca_cert.connect(
-            self._download_ca_cert)
-        self._provider_bootstrapper.check_ca_fingerprint.connect(
-            self._check_ca_fingerprint)
-        self._provider_bootstrapper.check_api_certificate.connect(
-            self._check_api_certificate)
+        self._backend = backend
+        self._backend_connect()
 
         self._domain = None
-        self._provider_config = ProviderConfig()
+
+        # this details are set when the provider download is complete.
+        self._provider_details = None
 
         # We will store a reference to the defers for eventual use
         # (eg, to cancel them) but not doing anything with them right now.
@@ -114,6 +100,8 @@ class Wizard(QtGui.QWizard):
         self.ui.lnProvider.textChanged.connect(self._enable_check)
         self.ui.rbNewProvider.toggled.connect(
             lambda x: self._enable_check())
+        self.ui.cbProviders.currentIndexChanged[int].connect(
+            self._reset_provider_check)
 
         self.ui.lblUser.returnPressed.connect(
             self._focus_password)
@@ -126,7 +114,7 @@ class Wizard(QtGui.QWizard):
 
         self.ui.rbExistingProvider.toggled.connect(self._skip_provider_checks)
 
-        usernameRe = QtCore.QRegExp(self.BARE_USERNAME_REGEX)
+        usernameRe = QtCore.QRegExp(USERNAME_REGEX)
         self.ui.lblUser.setValidator(
             QtGui.QRegExpValidator(usernameRe, self))
 
@@ -154,8 +142,7 @@ class Wizard(QtGui.QWizard):
     @QtCore.Slot()
     def _wizard_finished(self):
         """
-        SLOT
-        TRIGGER:
+        TRIGGERS:
             self.finished
 
         This method is called when the wizard is accepted or rejected.
@@ -166,6 +153,7 @@ class Wizard(QtGui.QWizard):
         self._provider_setup_ok = False
         self.ui.lnProvider.setText('')
         self.ui.grpCheckProvider.setVisible(False)
+        self._backend_disconnect()
 
     def _load_configured_providers(self):
         """
@@ -199,6 +187,10 @@ class Wizard(QtGui.QWizard):
             random.shuffle(pinned)  # don't prioritize alphabetically
             self.ui.cbProviders.addItems(pinned)
 
+        # We have configured providers, so by default we select the
+        # 'Use existing provider' option.
+        self.ui.rbExistingProvider.setChecked(True)
+
     def get_domain(self):
         return self._domain
 
@@ -214,23 +206,33 @@ class Wizard(QtGui.QWizard):
     def get_services(self):
         return self._selected_services
 
-    @QtCore.Slot()
+    @QtCore.Slot(unicode)
     def _enable_check(self, reset=True):
         """
-        SLOT
-        TRIGGER:
+        TRIGGERS:
             self.ui.lnProvider.textChanged
 
         Enables/disables the 'check' button in the SELECT_PROVIDER_PAGE
         depending on the lnProvider content.
+
+        :param reset: this contains the text of the line edit, and when is
+                      called directly defines whether we want to reset the
+                      checks.
+        :type reset: unicode or bool
         """
         enabled = len(self.ui.lnProvider.text()) != 0
-        enabled = enabled and self.ui.rbNewProvider.isChecked()
+        enabled = enabled or self.ui.rbExistingProvider.isChecked()
         self.ui.btnCheck.setEnabled(enabled)
 
         if reset:
             self._reset_provider_check()
 
+    def _focus_username(self):
+        """
+        Focus at the username lineedit for the registration page
+        """
+        self.ui.lblUser.setFocus()
+
     def _focus_password(self):
         """
         Focuses at the password lineedit for the registration page
@@ -253,21 +255,22 @@ class Wizard(QtGui.QWizard):
         password = self.ui.lblPassword.text()
         password2 = self.ui.lblPassword2.text()
 
-        ok, msg = basic_password_checks(username, password, password2)
-        if ok:
-            register = SRPRegister(provider_config=self._provider_config)
-            register.registration_finished.connect(
-                self._registration_finished)
+        user_ok, msg = username_checks(username)
+        if user_ok:
+            pass_ok, msg = password_checks(username, password, password2)
 
-            threads.deferToThread(
-                partial(register.register_user, username, password))
+        if user_ok and pass_ok:
+            self._set_register_status(self.tr("Starting registration..."))
 
+            self._backend.user_register(self._domain, username, password)
             self._username = username
             self._password = password
-            self._set_register_status(self.tr("Starting registration..."))
         else:
+            if user_ok:
+                self._focus_password()
+            else:
+                self._focus_username()
             self._set_register_status(msg, error=True)
-            self._focus_password()
             self.ui.btnRegister.setEnabled(True)
 
     def _set_registration_fields_visibility(self, visible):
@@ -291,42 +294,59 @@ class Wizard(QtGui.QWizard):
         # register button
         self.ui.btnRegister.setVisible(visible)
 
-    def _registration_finished(self, ok, req):
-        if ok:
-            user_domain = self._username + "@" + self._domain
-            message = "<font color='green'><h3>"
-            message += self.tr("User %s successfully registered.") % (
-                user_domain, )
-            message += "</h3></font>"
-            self._set_register_status(message)
+    @QtCore.Slot()
+    def _registration_finished(self):
+        """
+        TRIGGERS:
+            self._backend.signaler.srp_registration_finished
 
-            self.ui.lblPassword2.clearFocus()
-            self._set_registration_fields_visibility(False)
+        The registration has finished successfully, so we do some final steps.
+        """
+        user_domain = self._username + "@" + self._domain
+        message = "<font color='green'><h3>"
+        message += self.tr("User %s successfully registered.") % (
+            user_domain, )
+        message += "</h3></font>"
+        self._set_register_status(message)
 
-            # Allow the user to remember his password
-            if has_keyring():
-                self.ui.chkRemember.setVisible(True)
-                self.ui.chkRemember.setEnabled(True)
+        self.ui.lblPassword2.clearFocus()
+        self._set_registration_fields_visibility(False)
 
-            self.page(self.REGISTER_USER_PAGE).set_completed()
-            self.button(QtGui.QWizard.BackButton).setEnabled(False)
-        else:
-            old_username = self._username
-            self._username = None
-            self._password = None
-            error_msg = self.tr("Something has gone wrong. "
-                                "Please try again.")
-            try:
-                content, _ = get_content(req)
-                json_content = json.loads(content)
-                error_msg = json_content.get("errors").get("login")[0]
-                if not error_msg.istitle():
-                    error_msg = "%s %s" % (old_username, error_msg)
-            except Exception as e:
-                logger.error("Unknown error: %r" % (e,))
+        # Allow the user to remember his password
+        if has_keyring():
+            self.ui.chkRemember.setVisible(True)
+            self.ui.chkRemember.setEnabled(True)
 
-            self._set_register_status(error_msg, error=True)
-            self.ui.btnRegister.setEnabled(True)
+        self.page(self.REGISTER_USER_PAGE).set_completed()
+        self.button(QtGui.QWizard.BackButton).setEnabled(False)
+
+    @QtCore.Slot()
+    def _registration_failed(self):
+        """
+        TRIGGERS:
+            self._backend.signaler.srp_registration_failed
+
+        The registration has failed, so we report the problem.
+        """
+        self._username = self._password = None
+
+        error_msg = self.tr("Something has gone wrong. Please try again.")
+        self._set_register_status(error_msg, error=True)
+        self.ui.btnRegister.setEnabled(True)
+
+    @QtCore.Slot()
+    def _registration_taken(self):
+        """
+        TRIGGERS:
+            self._backend.signaler.srp_registration_taken
+
+        The requested username is taken, warn the user about that.
+        """
+        self._username = self._password = None
+
+        error_msg = self.tr("The requested username is taken, choose another.")
+        self._set_register_status(error_msg, error=True)
+        self.ui.btnRegister.setEnabled(True)
 
     def _set_register_status(self, status, error=False):
         """
@@ -350,7 +370,8 @@ class Wizard(QtGui.QWizard):
         self.ui.lblProviderSelectStatus.setText("")
         self._domain = None
         self.button(QtGui.QWizard.NextButton).setEnabled(False)
-        self.page(self.SELECT_PROVIDER_PAGE).set_completed(False)
+        self.page(self.SELECT_PROVIDER_PAGE).set_completed(
+            flags.SKIP_WIZARD_CHECKS)
 
     def _reset_provider_setup(self):
         """
@@ -360,17 +381,19 @@ class Wizard(QtGui.QWizard):
         self.ui.lblCheckCaFpr.setPixmap(None)
         self.ui.lblCheckApiCert.setPixmap(None)
 
+    @QtCore.Slot()
     def _check_provider(self):
         """
-        SLOT
         TRIGGERS:
-          self.ui.btnCheck.clicked
-          self.ui.lnProvider.returnPressed
+            self.ui.btnCheck.clicked
+            self.ui.lnProvider.returnPressed
 
         Starts the checks for a given provider
         """
-        if len(self.ui.lnProvider.text()) == 0:
-            return
+        if self.ui.rbNewProvider.isChecked():
+            self._domain = self.ui.lnProvider.text()
+        else:
+            self._domain = self.ui.cbProviders.currentText()
 
         self._provider_checks_ok = False
 
@@ -380,18 +403,23 @@ class Wizard(QtGui.QWizard):
 
         self.ui.grpCheckProvider.setVisible(True)
         self.ui.btnCheck.setEnabled(False)
-        self.ui.lnProvider.setEnabled(False)
+
+        # Disable provider widget
+        if self.ui.rbNewProvider.isChecked():
+            self.ui.lnProvider.setEnabled(False)
+        else:
+            self.ui.cbProviders.setEnabled(False)
+
         self.button(QtGui.QWizard.BackButton).clearFocus()
-        self._domain = self.ui.lnProvider.text()
 
         self.ui.lblNameResolution.setPixmap(self.QUESTION_ICON)
-        self._provider_select_defer = self._provider_bootstrapper.\
-            run_provider_select_checks(self._domain)
+        self._provider_select_defer = self._backend.\
+            provider_setup(self._domain)
 
+    @QtCore.Slot(bool)
     def _skip_provider_checks(self, skip):
         """
-        SLOT
-        Triggered:
+        TRIGGERS:
             self.ui.rbExistingProvider.toggled
 
         Allows the user to move to the next page without make any checks,
@@ -403,8 +431,6 @@ class Wizard(QtGui.QWizard):
         if skip:
             self._reset_provider_check()
 
-        self.page(self.SELECT_PROVIDER_PAGE).set_completed(skip)
-        self.button(QtGui.QWizard.NextButton).setEnabled(skip)
         self._use_existing_provider = skip
 
     def _complete_task(self, data, label, complete=False, complete_page=-1):
@@ -423,8 +449,8 @@ class Wizard(QtGui.QWizard):
         :param complete_page: page id to complete
         :type complete_page: int
         """
-        passed = data[self._provider_bootstrapper.PASSED_KEY]
-        error = data[self._provider_bootstrapper.ERROR_KEY]
+        passed = data[self._backend.PASSED_KEY]
+        error = data[self._backend.ERROR_KEY]
         if passed:
             label.setPixmap(self.OK_ICON)
             if complete:
@@ -434,16 +460,17 @@ class Wizard(QtGui.QWizard):
             label.setPixmap(self.ERROR_ICON)
             logger.error(error)
 
+    @QtCore.Slot(dict)
     def _name_resolution(self, data):
         """
-        SLOT
-        TRIGGER: self._provider_bootstrapper.name_resolution
+        TRIGGERS:
+            self._backend.signaler.prov_name_resolution
 
         Sets the status for the name resolution check
         """
         self._complete_task(data, self.ui.lblNameResolution)
         status = ""
-        passed = data[self._provider_bootstrapper.PASSED_KEY]
+        passed = data[self._backend.PASSED_KEY]
         if not passed:
             status = self.tr("<font color='red'><b>Non-existent "
                              "provider</b></font>")
@@ -453,85 +480,104 @@ class Wizard(QtGui.QWizard):
         self.ui.btnCheck.setEnabled(not passed)
         self.ui.lnProvider.setEnabled(not passed)
 
+    @QtCore.Slot(dict)
     def _https_connection(self, data):
         """
-        SLOT
-        TRIGGER: self._provider_bootstrapper.https_connection
+        TRIGGERS:
+            self._backend.signaler.prov_https_connection
 
         Sets the status for the https connection check
         """
         self._complete_task(data, self.ui.lblHTTPS)
         status = ""
-        passed = data[self._provider_bootstrapper.PASSED_KEY]
+        passed = data[self._backend.PASSED_KEY]
         if not passed:
             status = self.tr("<font color='red'><b>%s</b></font>") \
-                % (data[self._provider_bootstrapper.ERROR_KEY])
+                % (data[self._backend.ERROR_KEY])
             self.ui.lblProviderSelectStatus.setText(status)
         else:
             self.ui.lblProviderInfo.setPixmap(self.QUESTION_ICON)
         self.ui.btnCheck.setEnabled(not passed)
         self.ui.lnProvider.setEnabled(not passed)
 
+    @QtCore.Slot(dict)
     def _download_provider_info(self, data):
         """
-        SLOT
-        TRIGGER: self._provider_bootstrapper.download_provider_info
+        TRIGGERS:
+            self._backend.signaler.prov_download_provider_info
 
         Sets the status for the provider information download
         check. Since this check is the last of this set, it also
         completes the page if passed
         """
-        if self._provider_config.load(os.path.join("leap",
-                                                   "providers",
-                                                   self._domain,
-                                                   "provider.json")):
+        if data[self._backend.PASSED_KEY]:
             self._complete_task(data, self.ui.lblProviderInfo,
                                 True, self.SELECT_PROVIDER_PAGE)
             self._provider_checks_ok = True
+            lang = QtCore.QLocale.system().name()
+            self._backend.provider_get_details(self._domain, lang)
         else:
             new_data = {
-                self._provider_bootstrapper.PASSED_KEY: False,
-                self._provider_bootstrapper.ERROR_KEY:
+                self._backend.PASSED_KEY: False,
+                self._backend.ERROR_KEY:
                 self.tr("Unable to load provider configuration")
             }
             self._complete_task(new_data, self.ui.lblProviderInfo)
 
         status = ""
-        if not data[self._provider_bootstrapper.PASSED_KEY]:
+        if not data[self._backend.PASSED_KEY]:
             status = self.tr("<font color='red'><b>Not a valid provider"
                              "</b></font>")
             self.ui.lblProviderSelectStatus.setText(status)
         self.ui.btnCheck.setEnabled(True)
-        self.ui.lnProvider.setEnabled(True)
 
+        # Enable provider widget
+        if self.ui.rbNewProvider.isChecked():
+            self.ui.lnProvider.setEnabled(True)
+        else:
+            self.ui.cbProviders.setEnabled(True)
+
+    @QtCore.Slot()
+    def _provider_get_details(self, details):
+        """
+        Set the details for the just downloaded provider.
+
+        :param details: the details of the provider.
+        :type details: ProviderConfigLight
+        """
+        self._provider_details = details
+
+    @QtCore.Slot(dict)
     def _download_ca_cert(self, data):
         """
-        SLOT
-        TRIGGER: self._provider_bootstrapper.download_ca_cert
+        TRIGGERS:
+            self._backend.signaler.prov_download_ca_cert
 
         Sets the status for the download of the CA certificate check
         """
         self._complete_task(data, self.ui.lblDownloadCaCert)
-        passed = data[self._provider_bootstrapper.PASSED_KEY]
+        passed = data[self._backend.PASSED_KEY]
         if passed:
             self.ui.lblCheckCaFpr.setPixmap(self.QUESTION_ICON)
 
+    @QtCore.Slot(dict)
     def _check_ca_fingerprint(self, data):
         """
-        SLOT
-        TRIGGER: self._provider_bootstrapper.check_ca_fingerprint
+        TRIGGERS:
+            self._backend.signaler.prov_check_ca_fingerprint
 
         Sets the status for the CA fingerprint check
         """
         self._complete_task(data, self.ui.lblCheckCaFpr)
-        passed = data[self._provider_bootstrapper.PASSED_KEY]
+        passed = data[self._backend.PASSED_KEY]
         if passed:
             self.ui.lblCheckApiCert.setPixmap(self.QUESTION_ICON)
 
+    @QtCore.Slot(dict)
     def _check_api_certificate(self, data):
         """
-        SLOT
-        TRIGGER: self._provider_bootstrapper.check_api_certificate
+        TRIGGERS:
+            self._backend.signaler.prov_check_api_certificate
 
         Sets the status for the API certificate check. Also finishes
         the provider bootstrapper thread since it's not needed anymore
@@ -541,10 +587,12 @@ class Wizard(QtGui.QWizard):
                             True, self.SETUP_PROVIDER_PAGE)
         self._provider_setup_ok = True
 
+    @QtCore.Slot(str, int)
     def _service_selection_changed(self, service, state):
         """
-        SLOT
-        TRIGGER: service_checkbox.stateChanged
+        TRIGGERS:
+            service_checkbox.stateChanged
+
         Adds the service to the state if the state is checked, removes
         it otherwise
 
@@ -566,11 +614,9 @@ class Wizard(QtGui.QWizard):
         the user to enable or disable.
         """
         self.ui.grpServices.setTitle(
-            self.tr("Services by %s") %
-            (self._provider_config.get_name(),))
+            self.tr("Services by {0}").format(self._provider_details.name))
 
-        services = get_supported(
-            self._provider_config.get_services())
+        services = get_supported(self._provider_details.services)
 
         for service in services:
             try:
@@ -589,14 +635,19 @@ class Wizard(QtGui.QWizard):
                     self.tr("Something went wrong while trying to "
                             "load service %s" % (service,)))
 
+    @QtCore.Slot(int)
     def _current_id_changed(self, pageId):
         """
-        SLOT
-        TRIGGER: self.currentIdChanged
+        TRIGGERS:
+            self.currentIdChanged
 
         Prepares the pages when they appear
+
+        :param pageId: the new current page id.
+        :type pageId: int
         """
         if pageId == self.SELECT_PROVIDER_PAGE:
+            self._clear_register_widgets()
             skip = self.ui.rbExistingProvider.isChecked()
             if not self._provider_checks_ok:
                 self._enable_check()
@@ -608,38 +659,31 @@ class Wizard(QtGui.QWizard):
             if not self._provider_setup_ok:
                 self._reset_provider_setup()
                 sub_title = self.tr("Gathering configuration options for {0}")
-                sub_title = sub_title.format(self._provider_config.get_name())
+                sub_title = sub_title.format(self._provider_details.name)
                 self.page(pageId).setSubTitle(sub_title)
                 self.ui.lblDownloadCaCert.setPixmap(self.QUESTION_ICON)
-                self._provider_setup_defer = self._provider_bootstrapper.\
-                    run_provider_setup_checks(self._provider_config)
+                self._provider_setup_defer = self._backend.\
+                    provider_bootstrap(self._domain)
 
         if pageId == self.PRESENT_PROVIDER_PAGE:
-            self.page(pageId).setSubTitle(self.tr("Description of services "
-                                                  "offered by %s") %
-                                          (self._provider_config
-                                           .get_name(),))
+            sub_title = self.tr("Description of services offered by {0}")
+            sub_title = sub_title.format(self._provider_details.name)
+            self.page(pageId).setSubTitle(sub_title)
 
-            lang = QtCore.QLocale.system().name()
-            self.ui.lblProviderName.setText(
-                "<b>%s</b>" %
-                (self._provider_config.get_name(lang=lang),))
-            self.ui.lblProviderURL.setText(
-                "https://%s" % (self._provider_config.get_domain(),))
-            self.ui.lblProviderDesc.setText(
-                "<i>%s</i>" %
-                (self._provider_config.get_description(lang=lang),))
-
-            self.ui.lblServicesOffered.setText(self._provider_config
-                                               .get_services_string())
-            self.ui.lblProviderPolicy.setText(self._provider_config
-                                              .get_enrollment_policy())
+            details = self._provider_details
+            name = "<b>{0}</b>".format(details.name)
+            domain = "https://{0}".format(details.domain)
+            description = "<i>{0}</i>".format(details.description)
+            self.ui.lblProviderName.setText(name)
+            self.ui.lblProviderURL.setText(domain)
+            self.ui.lblProviderDesc.setText(description)
+            self.ui.lblServicesOffered.setText(details.services_string)
+            self.ui.lblProviderPolicy.setText(details.enrollment_policy)
 
         if pageId == self.REGISTER_USER_PAGE:
-            self.page(pageId).setSubTitle(self.tr("Register a new user with "
-                                                  "%s") %
-                                          (self._provider_config
-                                           .get_name(),))
+            sub_title = self.tr("Register a new user with {0}")
+            sub_title = sub_title.format(self._provider_details.name)
+            self.page(pageId).setSubTitle(sub_title)
             self.ui.chkRemember.setVisible(False)
 
         if pageId == self.SERVICES_PAGE:
@@ -662,11 +706,57 @@ class Wizard(QtGui.QWizard):
         if self.currentPage() == self.page(self.SELECT_PROVIDER_PAGE):
             if self._use_existing_provider:
                 self._domain = self.ui.cbProviders.currentText()
-                self._provider_config = ProviderConfig.get_provider_config(
-                    self._domain)
                 if self._show_register:
                     return self.REGISTER_USER_PAGE
                 else:
                     return self.SERVICES_PAGE
 
         return QtGui.QWizard.nextId(self)
+
+    def _clear_register_widgets(self):
+        """
+        Clears the widgets that my be filled and a possible error message.
+        """
+        self._set_register_status("")
+        self.ui.lblUser.setText("")
+        self.ui.lblPassword.setText("")
+        self.ui.lblPassword2.setText("")
+
+    def _backend_connect(self):
+        """
+        Connects all the backend signals with the wizard.
+        """
+        sig = self._backend.signaler
+        sig.prov_name_resolution.connect(self._name_resolution)
+        sig.prov_https_connection.connect(self._https_connection)
+        sig.prov_download_provider_info.connect(self._download_provider_info)
+        sig.prov_get_details.connect(self._provider_get_details)
+
+        sig.prov_download_ca_cert.connect(self._download_ca_cert)
+        sig.prov_check_ca_fingerprint.connect(self._check_ca_fingerprint)
+        sig.prov_check_api_certificate.connect(self._check_api_certificate)
+
+        sig.srp_registration_finished.connect(self._registration_finished)
+        sig.srp_registration_failed.connect(self._registration_failed)
+        sig.srp_registration_taken.connect(self._registration_taken)
+
+    def _backend_disconnect(self):
+        """
+        This method is called when the wizard dialog is closed.
+        We disconnect all the backend signals in here.
+        """
+        sig = self._backend.signaler
+        try:
+            # disconnect backend signals
+            sig.prov_name_resolution.disconnect(self._name_resolution)
+            sig.prov_https_connection.disconnect(self._https_connection)
+            sig.prov_download_provider_info.disconnect(
+                self._download_provider_info)
+
+            sig.prov_download_ca_cert.disconnect(self._download_ca_cert)
+            sig.prov_check_ca_fingerprint.disconnect(
+                self._check_ca_fingerprint)
+            sig.prov_check_api_certificate.disconnect(
+                self._check_api_certificate)
+        except RuntimeError:
+            pass  # Signal was not connected
